@@ -19,6 +19,7 @@ from decimal import ROUND_HALF_UP
 from oslo_config import cfg
 from oslo_log import log
 from voluptuous import In
+from voluptuous import Optional
 from voluptuous import Required
 from voluptuous import Schema
 
@@ -26,7 +27,9 @@ from cloudkitty import collector
 from cloudkitty.collector.exceptions import CollectError
 from cloudkitty.common.prometheus_client import PrometheusClient
 from cloudkitty.common.prometheus_client import PrometheusResponseError
+from cloudkitty import dataframe
 from cloudkitty import utils as ck_utils
+from cloudkitty.utils import tz as tzutils
 
 
 LOG = log.getLogger(__name__)
@@ -62,13 +65,25 @@ cfg.CONF.register_opts(collector_prometheus_opts, PROMETHEUS_COLLECTOR_OPTS)
 CONF = cfg.CONF
 
 PROMETHEUS_EXTRA_SCHEMA = {
-    Required('extra_args'): {
+    Required('extra_args', default={}): {
         Required('aggregation_method', default='max'):
             In([
                 'avg', 'count', 'max',
                 'min', 'stddev', 'stdvar',
                 'sum'
             ]),
+        Optional('query_function'):
+            In([
+                'abs', 'ceil', 'exp',
+                'floor', 'ln', 'log2',
+                'log10', 'round', 'sqrt'
+            ]),
+        Optional('range_function'):
+            In([
+                'changes', 'delta', 'deriv',
+                'idelta', 'irange', 'irate',
+                'rate'
+            ])
     }
 }
 
@@ -76,8 +91,8 @@ PROMETHEUS_EXTRA_SCHEMA = {
 class PrometheusCollector(collector.BaseCollector):
     collector_name = 'prometheus'
 
-    def __init__(self, transformers, **kwargs):
-        super(PrometheusCollector, self).__init__(transformers, **kwargs)
+    def __init__(self, **kwargs):
+        super(PrometheusCollector, self).__init__(**kwargs)
         url = CONF.collector_prometheus.prometheus_url
 
         user = CONF.collector_prometheus.prometheus_user
@@ -137,24 +152,55 @@ class PrometheusCollector(collector.BaseCollector):
         """Returns metrics to be valorized."""
         scope_key = CONF.collect.scope_key
         method = self.conf[metric_name]['extra_args']['aggregation_method']
+        query_function = self.conf[metric_name]['extra_args'].get(
+            'query_function')
+        range_function = self.conf[metric_name]['extra_args'].get(
+            'range_function')
         groupby = self.conf[metric_name].get('groupby', [])
         metadata = self.conf[metric_name].get('metadata', [])
-        period = end - start
+        period = tzutils.diff_seconds(end, start)
         time = end
 
-        query = '{0}({0}_over_time({1}{{{2}="{3}"}}[{4}s])) by ({5})'.format(
-            method,
+        # The metric with the period
+        query = '{0}{{{1}="{2}"}}[{3}s]'.format(
             metric_name,
             scope_key,
             scope_id,
-            period,
-            ', '.join(groupby + metadata),
+            period
+        )
+        # Applying the aggregation_method or the range_function on
+        # a Range Vector
+        if range_function is not None:
+            query = "{0}({1})".format(
+                range_function,
+                query
+            )
+        else:
+            query = "{0}_over_time({1})".format(
+                method,
+                query
+            )
+        # Applying the query_function
+        if query_function is not None:
+            query = "{0}({1})".format(
+                query_function,
+                query
+            )
+        # Applying the aggregation_method on a Instant Vector
+        query = "{0}({1})".format(
+            method,
+            query
+        )
+        # Filter by groupby and metadata
+        query = "{0} by ({1})".format(
+            query,
+            ', '.join(groupby + metadata)
         )
 
         try:
             res = self._conn.get_instant(
                 query,
-                time,
+                time.isoformat(),
             )
         except PrometheusResponseError as e:
             raise CollectError(*e.args)
@@ -176,13 +222,12 @@ class PrometheusCollector(collector.BaseCollector):
                 item,
             )
 
-            item = self.t_cloudkitty.format_item(
+            formatted_resources.append(dataframe.DataPoint(
+                self.conf[metric_name]['unit'],
+                qty,
+                0,
                 groupby,
                 metadata,
-                self.conf[metric_name]['unit'],
-                qty=qty,
-            )
-
-            formatted_resources.append(item)
+            ))
 
         return formatted_resources
